<?php
namespace nulib\schema\types;

use nulib\schema\_scalar\ScalarSchema;
use nulib\schema\Result;
use nulib\schema\Schema;

class tpkey extends _tunion {
  const NAME = "pkey";

  static function ensure_pkey(&$pkey): void {
    if ($pkey === null) $pkey = "";
    elseif ($pkey === false) $pkey = 0;
    elseif (!is_string($pkey) && !is_int($pkey) && !is_array($pkey)) $pkey = strval($pkey);
    if (is_array($pkey)) {
      foreach ($pkey as &$key) {
        tkey::ensure_key($key);
      }; unset($key);
    }
  }

  static function ensure_npkey(&$pkey): void {
    if ($pkey !== null) self::ensure_pkey($pkey);
  }

  function getClass(): string {
    return "string|int|array";
  }

  function getNullValue() {
    return $this->nullable? null: [];
  }

  function isValid($value, ?bool &$normalized=null): bool {
    $normalized = is_string($value) || is_int($value) || is_array($value);
    return $normalized || is_scalar($value);
  }

  function parse(string $value) {
    return $value;
  }

  /**
   * @var ScalarSchema $schema
   */
  function normalize(&$value, Result $result, Schema $schema): bool {
    if ($result->normalized) {
    } elseif (is_string($value) || is_int($value) || is_array($value)) {
      $result->setNormalized();
    } elseif (is_scalar($value)) {
      $value = strval($value);
      return true;
    }
    return false;
  }

  function format($value, $format=null): string {
    if (is_array($value)) return implode(".", $value);
    else return strval($value);
  }
}
